//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionBytes.h>
#import <Diffusion/PTDiffusionTopicsFeature.h>

@class PTDiffusionBinaryDelta;

@protocol PTDiffusionBinaryValueStreamDelegate;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief An immutable binary value with support for binary deltas.

 @since 5.9
 */
@interface PTDiffusionBinary : PTDiffusionBytes

/**
 Apply a binary delta to the receiver's binary value to create a new value.

 @param delta The delta describing the changes to be applied to the receiver's
 value in order to create the value to be returned.

 @param error Location to store a reason if this method returns `nil` to
 indicate failure.

 @return The new binary value or `nil` if there was a failure when applying the
 delta (e.g. either the delta is invalid).

 @exception NSInvalidArgumentException Raised if the delta argument is `nil`.

 @since 5.9
 */
-(nullable PTDiffusionBinary *)applyDelta:(PTDiffusionBinaryDelta *)delta
                                    error:(NSError **)error;

/**
 Compare the receiver's binary value with another (e.g. an earlier version) to
 create a binary delta.

 @param binary The original to which the delta should be able to be applied in
 order to generate the value represented by the receiver.

 @return A binary delta representing the difference between binary and the
 receiver.

 @exception NSInvalidArgumentException Raised if the binary argument is `nil`.

 @since 5.9
 */
-(PTDiffusionBinaryDelta *)diffFromBinary:(PTDiffusionBinary *)binary;

/**
 Creates a value stream capable of receiving binary values.

 @param delegate The object which will handle the incoming stream. A weak
 reference is maintained to this object by the value stream.

 @return A value stream associated with the supplied delegate.

 @exception NSInvalidArgumentException Raised if the delegate argument is `nil`.

 @since 5.9
 */
+(PTDiffusionValueStream *)valueStreamWithDelegate:(id<PTDiffusionBinaryValueStreamDelegate>)delegate;

/**
 Compares the receiver to the given binary.

 @param binary The binary object with which to compare the receiver.

 @return `YES` if the data in binary is equal to the contents of the receiver, otherwise `NO`.

 @note Two binary objects are equal if they hold the same number of data bytes, and if the
 bytes at the same position in the objects are the same.

 @since 5.9
 */
-(BOOL)isEqualToBinary:(nullable PTDiffusionBinary *)binary;

@end

NS_ASSUME_NONNULL_END
