//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

@class PTDiffusionRecordContentSchema;
@class PTDiffusionRecord;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Record content is a collection of records.
 
 A record content object is immutable.
 
 @see PTDiffusionContent
 @see PTDiffusionRecordContentSchema

 @since 5.6
 */
@interface PTDiffusionRecordContent : NSObject <NSCopying>

-(instancetype)init NS_UNAVAILABLE;

/**
 The data defining the receiver.
 This returns the raw underlying bytes. It has not been transformed in any way.

 @since 5.6
 */
@property(nonatomic, readonly, copy) NSData* data;

/**
 The schema defining the transformation applied to the receiver's data to produce the records.

 @since 5.6
 */
@property(nonatomic, readonly, copy, nullable) PTDiffusionRecordContentSchema* schema;

/**
 Returns records transformed by a PTDiffusionRecordMetadata object with the given name.

 If no records were found with the given name or schema is `nil` then this method will return
 an empty array.

 @param name The name to find when examining metadata to select records.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
-(NSArray<PTDiffusionRecord *> *)recordsWithName:(NSString *)name;

/**
 Returns the first record object transformed by a PTDiffusionRecordMetadata object with the given name.

 If no records were found with the given name or schema is `nil` then this method will return `nil`.

 @param name The name to find when examining metadata to select record.

 @exception NSInvalidArgumentException If the given name is `nil`.

 @since 5.6
 */
-(nullable PTDiffusionRecord *)recordWithName:(NSString *)name;

/**
 Returns all records encapsulated by the receiver. This array may be empty.

 @since 5.6
 */
@property(nonatomic, readonly) NSArray<PTDiffusionRecord *> * records;

/**
 Compares the receiver to the given record content.

 @param recordContent The record content object with which to compare the receiver.

 @return `YES` if the record content is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToRecordContent:(nullable PTDiffusionRecordContent *)recordContent;

@end

NS_ASSUME_NONNULL_END
