//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionStreamDelegate.h>
#import <Diffusion/PTDiffusionTopicUnsubscriptionReason.h>

@class PTDiffusionContent;
@class PTDiffusionStream;
@class PTDiffusionTopicDetails;
@class PTDiffusionUpdateContext;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief The topic stream delegate protocol defines the methods to be implemented by classes
 wishing to receive streamed topic updates.

 @see PTDiffusionTopicsFeature

 @since 5.6
 */
@protocol PTDiffusionTopicStreamDelegate <PTDiffusionStreamDelegate>

@required

/**
 An update was received for a topic path handled by the topic stream.

 @param stream The topic stream that received the update.

 @param topicPath The topic path which was updated.

 @param content The content of the updated topic.

 @param context The context of the update.

 @since 5.6
 */
-(void)diffusionStream:(PTDiffusionStream *)stream
    didUpdateTopicPath:(NSString *)topicPath
               content:(PTDiffusionContent *)content
               context:(PTDiffusionUpdateContext *)context;

@optional

/**
 A subscription notification was received for a topic path handled by the topic stream.

 The subscription could be as a result of a request from this client's associated session or it
 might have been requested by another control client.

 @param stream The topic stream that received the subscription notification.

 @param topicPath The topic path to which the session has been subscribed.

 @param details The details of the subscription.

 @since 5.6
 */
-(void)     diffusionStream:(PTDiffusionStream *)stream
    didSubscribeToTopicPath:(NSString *)topicPath
                    details:(PTDiffusionTopicDetails *)details;

/**
 An unsubscription notification was received for a topic path handled by the topic stream.

 The unsubscription could be as a result of a request from this client's associated session or it
 might have been requested by another control client.

 @param stream The topic stream that received the unsubscription notification.

 @param topicPath The topic path from which the session has been unsubscribed.

 @param reason The reason for the unsubscription.

 @since 5.7
 */
-(void)         diffusionStream:(PTDiffusionStream *)stream
    didUnsubscribeFromTopicPath:(NSString *)topicPath
                         reason:(PTDiffusionTopicUnsubscriptionReason)reason;

/**
 An unsubscription notification was received for a topic path handled by the topic stream when
 the newer diffusionStream:didUnsubscribeFromTopicPath:reason: method has not been defined
 on the delegate.

 @param stream The topic stream that received the unsubscription notification.

 @param topicPath The topic path from which the session has been unsubscribed.

 @param reason The reason for the unsubscription.

 @see diffusionStream:didUnsubscribeFromTopicPath:reason:

 @since 5.6

 @deprecated Since 5.7 as this method was misnamed in the original 5.6 release.
 */
-(void)    diffusionTopicStream:(PTDiffusionStream *)stream
    didUnsubscribeFromTopicPath:(NSString *)topicPath
                         reason:(PTDiffusionTopicUnsubscriptionReason)reason __deprecated;

@end


NS_ASSUME_NONNULL_END
